@@ -1,11 +1,8 @@
 // Copyright (C) 2021 Scott Lamb <slamb@slamb.org>
 // SPDX-License-Identifier: MIT OR Apache-2.0
 
-use failure::{bail, format_err, Error};
-use std::{
-    convert::TryFrom,
-    num::{NonZeroI32, NonZeroU32},
-};
+use std::convert::TryFrom;
+use std::num::{NonZeroI32, NonZeroU32};
 
 use crate::Timestamp;
 
@@ -34,19 +31,19 @@ impl Timeline {
         start: Option<u32>,
         clock_rate: u32,
         enforce_with_max_forward_jump_secs: Option<NonZeroU32>,
-    ) -> Result<Self, Error> {
+    ) -> Result<Self, String> {
         let clock_rate = NonZeroU32::new(clock_rate)
-            .ok_or_else(|| format_err!("clock_rate=0 rejected to prevent division by zero"))?;
-        let max_forward_jump =
-            enforce_with_max_forward_jump_secs
-                .map(|j| i32::try_from(u64::from(j.get()) * u64::from(clock_rate.get())))
-                .transpose()
-                .map_err(|_| {
-                    format_err!(
-                "clock_rate={} rejected because max forward jump of {} sec exceeds i32::MAX",
-                clock_rate, MAX_FORWARD_TIME_JUMP_SECS)
-                })?
-                .map(|j| NonZeroI32::new(j).expect("non-zero times non-zero must be non-zero"));
+            .ok_or_else(|| "clock_rate=0 rejected to prevent division by zero".to_string())?;
+        let max_forward_jump = enforce_with_max_forward_jump_secs
+            .map(|j| i32::try_from(u64::from(j.get()) * u64::from(clock_rate.get())))
+            .transpose()
+            .map_err(|_| {
+                format!(
+                    "clock_rate={} rejected because max forward jump of {} sec exceeds i32::MAX",
+                    clock_rate, MAX_FORWARD_TIME_JUMP_SECS
+                )
+            })?
+            .map(|j| NonZeroI32::new(j).expect("non-zero times non-zero must be non-zero"));
         Ok(Timeline {
             timestamp: i64::from(start.unwrap_or(0)),
             start,
@@ -62,10 +59,10 @@ impl Timeline {
     ///
     /// If enforcement was enabled, this produces a monotonically increasing
     /// [Timestamp], erroring on excessive or backward time jumps.
-    pub fn advance_to(&mut self, rtp_timestamp: u32) -> Result<Timestamp, Error> {
+    pub fn advance_to(&mut self, rtp_timestamp: u32) -> Result<Timestamp, String> {
         let (timestamp, delta) = self.ts_and_delta(rtp_timestamp)?;
         if matches!(self.max_forward_jump, Some(j) if !(0..j.get()).contains(&delta)) {
-            bail!(
+            return Err(format!(
                 "Timestamp jumped {} ({:.03} sec) from {} to {}; \
                    policy is to allow 0..{} sec only",
                 delta,
@@ -73,7 +70,7 @@ impl Timeline {
                 self.timestamp,
                 timestamp,
                 self.max_forward_jump_secs
-            );
+            ));
         }
         self.timestamp = timestamp.timestamp;
         Ok(timestamp)
@@ -84,11 +81,11 @@ impl Timeline {
     ///
     /// This is useful for RTP timestamps in RTCP packets. They commonly refer
     /// to time slightly before the most timestamp of the matching RTP stream.
-    pub fn place(&mut self, rtp_timestamp: u32) -> Result<Timestamp, Error> {
+    pub fn place(&mut self, rtp_timestamp: u32) -> Result<Timestamp, String> {
         Ok(self.ts_and_delta(rtp_timestamp)?.0)
     }
 
-    fn ts_and_delta(&mut self, rtp_timestamp: u32) -> Result<(Timestamp, i32), Error> {
+    fn ts_and_delta(&mut self, rtp_timestamp: u32) -> Result<(Timestamp, i32), String> {
         let start = match self.start {
             None => {
                 self.start = Some(rtp_timestamp);
@@ -106,21 +103,18 @@ impl Timeline {
                 // take ~2^31 packets (~ 4 billion) to advance the time this far
                 // forward or backward even with no limits on time jump per
                 // packet.
-                format_err!(
+                format!(
                     "timestamp {} + delta {} won't fit in i64!",
-                    self.timestamp,
-                    delta
+                    self.timestamp, delta
                 )
             })?;
 
         // Also error in similarly-unlikely NPT underflow.
         if timestamp.checked_sub(i64::from(start)).is_none() {
-            bail!(
+            return Err(format!(
                 "timestamp {} + delta {} - start {} underflows i64!",
-                self.timestamp,
-                delta,
-                start
-            );
+                self.timestamp, delta, start
+            ));
         }
         Ok((
             Timestamp {
