@@ -81,7 +81,7 @@ impl AudioSpecificConfig {
             o => o,
         };
 
-        // ISO/IEC 14496-3 section 1.6.3.4.
+        // ISO/IEC 14496-3 section 1.6.3.3.
         let sampling_frequency = match r
             .read_u8(4)
             .map_err(|e| format!("unable to read sampling_frequency: {}", e))?
@@ -90,6 +90,7 @@ impl AudioSpecificConfig {
             0x1 => 88_200,
             0x2 => 64_000,
             0x3 => 48_000,
+            0x4 => 44_100,
             0x5 => 32_000,
             0x6 => 24_000,
             0x7 => 22_050,
@@ -104,7 +105,7 @@ impl AudioSpecificConfig {
             0xf => r
                 .read_u32(24)
                 .map_err(|e| format!("unable to read sampling_frequency ext: {}", e))?,
-            _ => unreachable!(),
+            0x10..=0xff => unreachable!(),
         };
         let channels = {
             let c = r
@@ -426,7 +427,7 @@ pub(crate) struct Depacketizer {
 
 #[derive(Debug)]
 struct Aggregate {
-    ctx: crate::RtspMessageContext,
+    ctx: crate::PacketContext,
 
     /// RTP packets lost before the next frame in this aggregate. Includes old
     /// loss that caused a previous fragment to be too short.
@@ -439,7 +440,6 @@ struct Aggregate {
     /// to be too short.
     loss_since_mark: bool,
 
-    channel_id: u8,
     stream_id: usize,
     ssrc: u32,
     sequence_number: u16,
@@ -605,7 +605,6 @@ impl Depacketizer {
                     ctx: pkt.ctx,
                     loss: *prev_loss + pkt.loss,
                     loss_since_mark: pkt.loss > 0,
-                    channel_id: pkt.channel_id,
                     stream_id: pkt.stream_id,
                     ssrc: pkt.ssrc,
                     sequence_number: pkt.sequence_number,
@@ -725,8 +724,7 @@ impl Depacketizer {
 fn error(conn_ctx: ConnectionContext, agg: Aggregate, description: String) -> Error {
     Error(Box::new(ErrorInt::RtpPacketError {
         conn_ctx,
-        msg_ctx: agg.ctx,
-        channel_id: agg.channel_id,
+        pkt_ctx: agg.ctx,
         stream_id: agg.stream_id,
         ssrc: agg.ssrc,
         sequence_number: agg.sequence_number,
